import { component, detectIsComponent, useMemo, currentFiberStore } from '@dark-engine/core';
const $$portal = Symbol('portal');
function createPortal(slot, container) {
  if (process.env.NODE_ENV !== 'production') {
    if (!(container instanceof Element)) {
      throw new Error(`[Dark]: createPortal receives only Element as container!`);
    }
  }
  return Portal({ container, slot });
}
const Portal = component(
  props => {
    const element = props.container;
    const fiber = currentFiberStore.get();
    useMemo(() => (element.textContent = ''), []);
    fiber.element = element;
    delete props.container;
    return props.slot;
  },
  { token: $$portal },
);
const detectIsPortal = instance => detectIsComponent(instance) && instance.token === $$portal;
const getPortalContainer = fiber => (detectIsPortal(fiber.inst) ? fiber.element : null);
function unmountPortal(fiber) {
  const element = getPortalContainer(fiber);
  element && (element.textContent = '');
}
export { createPortal, detectIsPortal, getPortalContainer, unmountPortal };
//# sourceMappingURL=portal.js.map
