import {
  ROOT,
  Fiber,
  EffectTag,
  platform,
  flatten,
  detectIsUndefined,
  TagVirtualNode,
  rootStore,
  wipRootStore,
  currentRootStore,
  nextUnitOfWorkStore,
  mountStore,
  TaskPriority,
  createReplacer,
  isInsertionEffectsZone,
  isLayoutEffectsZone,
  isHydrateZone,
} from '@dark-engine/core';
import { createNativeElement, applyCommit, finishCommitWork } from '../dom';
import { detectIsPortal, unmountPortal } from '../portal';
import { scheduleCallback, shouldYeildToHost } from '../scheduler';
let isInjected = false;
const roots = new Map();
function inject() {
  platform.createElement = createNativeElement;
  platform.raf = requestAnimationFrame.bind(this);
  platform.caf = cancelAnimationFrame.bind(this);
  platform.schedule = scheduleCallback;
  platform.shouldYeild = shouldYeildToHost;
  platform.commit = applyCommit;
  platform.finishCommit = finishCommitWork;
  platform.detectIsDynamic = () => true;
  platform.detectIsPortal = detectIsPortal;
  platform.unmountPortal = unmountPortal;
  platform.restart = () => {};
  isInjected = true;
}
function render(element, container, hydrate = false) {
  !isInjected && inject();
  if (process.env.NODE_ENV !== 'production') {
    if (!(container instanceof Element)) {
      throw new Error(`[Dark]: render receives only Element as container!`);
    }
  }
  const isMounted = !detectIsUndefined(roots.get(container));
  let rootId = null;
  if (!isMounted) {
    rootId = roots.size;
    roots.set(container, rootId);
    if (!hydrate) {
      container.innerHTML = '';
    }
  } else {
    rootId = roots.get(container);
  }
  if (isInsertionEffectsZone.get(rootId)) return;
  const callback = () => {
    rootStore.set(rootId);
    const currentRoot = currentRootStore.get();
    const isUpdate = Boolean(currentRoot);
    const fiber = new Fiber().mutate({
      element: container,
      inst: new TagVirtualNode(ROOT, {}, flatten([element || createReplacer()])),
      alt: currentRoot,
      tag: isUpdate ? EffectTag.U : EffectTag.C,
    });
    mountStore.reset();
    wipRootStore.set(fiber);
    isHydrateZone.set(hydrate);
    nextUnitOfWorkStore.set(fiber);
  };
  platform.schedule(callback, {
    priority: TaskPriority.NORMAL,
    forceSync: hydrate || isLayoutEffectsZone.get(),
  });
}
export { render, roots };
//# sourceMappingURL=render.js.map
